/***  Layered.t version 1.1 ***/
/* A layered clothing library for TADS 3 */
/* Copyright 2007 by Sir Gareth */
/* The author would like to thank Newkid, Choices, MMXTads, and */
/* Scarlet Herring, who provided inspiration for parts of this library. */


/* helper functions; do these already exist? */

saylist(l)  /* prints out list of objects in list l */
{
  if(l.length() > 0)
  {
   foreach(local cur in l)
   {
      "<<cur.theName>> ";
   }
  }
  else "nothing";
}

isMemberOfList(a, l)  /* returns true if a is a member of the list l */
{
   foreach(local cur in l)
   {
      if(a == cur) return true;
   }
   return nil;
}

/* Material for clothes */

thinCloth:  Material
   seeThru = opaque
   hearThru = transparent
   smellThru = transparent
   touchThru = opaque
;

seeThruCloth:  Material
   seeThru = transparent
   hearThru = transparent
   smellThru = transparent
   touchThru = opaque
;

/* Fixed clothing which can't be removed */
class FixedClothing: Wearable
  noDoffMsg = "That probably wouldn't be appropriate. "
  dobjFor(Doff)
  {
    verify() {}
    check() { noDoffMsg; exit; }
  }
;


/* Layered clothing */
/* Main two classes are Layered (layered clothing) and BodyArea (a component of an actor that can be clothed) */

class ClothedItem: Thing
    senseUnderClothes(sense) { return nil; }   /* default is object can't be sensed if under opaque clothing */
    concealer(sense)       /* This is separately defined in the subclasses Layered and BodyArea */
    {
      return nil;
    }
    /* can the object be sensed? */
    canBeSensed(sense, info, pov)
    {
      if(concealer(sense) && !senseUnderClothes(sense)) return nil;
      else return inherited(sense, info, pov);
    }
;


modify Actor
   clothingSize = 0
;

/*  A ClothedPerson can dress and undress using a predefined list of clothing. */
/*  A Person does not need to be defined as a ClothedPerson in */
/*  order to wear Layered clothes */
class ClothedPerson: Person
  /* list of clothing to put on when getting dressed */
  /* should be listed in order actor will put them on */
  /* ie starting with underwear and progressing outwards */
  clothingList = []  
  willDress = nil
  dressMsg = "{You/he} start{s} to get dressed. "
  willUndress = nil
  undressMsg = "{You/he} take{s} off all of {your} clothing. "
  listClothing
  {
      local l = [];
      if(contents.length() == 0) return l;
      foreach(local cur in contents)
      {
        if(cur.ofKind(Layered) && cur.isWorn())
        {
          l += cur;
        }
      }
      return l;
  }
  quickDress    /* used to insta-clothe the actor, e.g. at the end of a scene */
  {
    if( clothingList.length() == 0 ) return nil;
    foreach(local cur in clothingList)
    {
      cur.moveInto(self);
      cur.makeWornBy(self);
    }
    return true;
  }
  quickUndress   /* used to insta-strip the actor */
  {
    local l = listClothing;
    foreach(local cur in l)
    {
        cur.makeWornBy(nil);
        if(!cur.neverCover) cur.noCover = nil;
        gActor.getDropDestination(cur, nil)
            .receiveDrop(cur, dropTypeDrop);     
    }
  }
  dobjFor(Undress)
  {
    verify() {}
    check()
    {
      "{You/he} need{s} to be more specific about what to remove. ";
      exit;
 
    }
  }
  dobjFor(Dress)
  {
    verify() {}
    check()
    {
      "{You/he} need{s} to be more specific about what to put on. ";
      exit;
    }
  }
  iobjFor(PutOn)
  {
    verify()
    {
      illogical( 'If {you/he} want{s} someone to wear {the dobj/him}, use the verb WEAR. ');
    }
  }
;

class Layered: ClothedItem, Wearable
    material = thinCloth   /* default: opaque to sight but can touch, hear, and smell through */
    layer = 0
    clothingSize
    {
      if(owner) return owner.clothingSize;   /* default is same size as owner */
      else return 0;
    }
    coverlist = []   /* The list of body areas this article covers */
    noCover = nil     /* If true, then it doesn't cover any body areas */
    neverCover = nil  /* If nil, then noCover will automatically be set to nil when doffed */
    wornDesc = "\^<<location.theName>> <<location.verbToBe>> wearing <<aName>>. "
    notWornDesc = nil    /* desc if not worn.  To customize, change this to text */
    noCoverDesc = "\^<<theName>> <<verbToBe>> opened up. "
    wearMsg = "{You/he} put{s} on <<theName>>. "
    doffMsg = "{You/he} take{s} off <<theName>>. "
    openMsg = "{You/he} opens{s} up <<theName>>. "
    tooBigMsg = "\^<<theName>> is too large for {you/him}. "
    tooSmallMsg = "\^<<theName>> is too small for {you/him}. "

   /*  Customize description depending on whether article is worn and/or open */
    desc
    {
      if(isWorn())
      {
        if(noCover) noCoverDesc;
        else wornDesc;
      }
      else if(notWornDesc) notWornDesc;
      else inherited();
    }
    
    /* helper description to be used in the desc of the actor wearing it */
    wornOnActorDesc
    {
      if(noCover) noCoverDesc;
      else wornDesc;
    }
    actorDesc(actor)
    {
      if(isWornBy(actor) && canBeSeen) wornOnActorDesc;
    }
    
    intersection(clothes)  /* return list composed of intersection of coverlists of self and clothes */
    {
      local l = [];
      if(clothes.coverlist.length() == 0 || coverlist.length() == 0) return l;
      foreach(local cur in clothes.coverlist)
      {
         if(isMemberOfList(cur, coverlist)) l += cur;
      }
      return l;
    }
    wornover(clothes)   /* return true if self must be removed before clothes */
    {
      if(intersection(clothes) != [] && layer >= clothes.layer) return true;
      else return nil;
    }
    covers(c)  /* return true if self covers clothes or body part */
    {
      if(c.ofKind(Layered))
      {
         if(intersection(c) == c.coverlist && layer > c.layer && !noCover) return true;
      }
      if(c.ofKind(BodyArea))
      {
         foreach(local cur in coverlist)
         {
            if(c.ofKind(cur)) return true;
         }
         return nil;
      }
      return nil;
    }
    conceals(c, sense)   /* return true if c can't be sensed through self */
    {
      if(covers(c) && material.senseThru(sense) != transparent) return true;
      else return nil;
    }
    blocker(actor)        /* Returns outermost clothing blocking this article from being donned or doffed */
    {
      local c = nil, l = -1;
      if(actor.contents.length() == 0) return nil;
      foreach(local cur in actor.contents)
      {
        if(cur.ofKind(Layered))
        {
          if(cur.wornover(self) && cur.layer > l && cur.isWorn() && cur != self)
          {
            c = cur;
            l = cur.layer;
          }
        }
      }
      return c;
    }
    cover          /* Returns outermost clothing covering this part */
    {
      local c = nil, l = -1;
      if( !isWorn() ) return nil;
      if(location.contents.length() == 0) return nil;
      foreach(local cur in location.contents)
      {
        if(cur.ofKind(Layered))
        {
          if(cur.covers(self) && cur.layer > l && cur.isWorn())
          {
            c = cur;
            l = cur.layer;
          }
        }
      }
      return c;
    }
    concealer(sense)   /* Returns outmost clothing concealing this article from being sensed */
    {
      local c = nil, l = -1;
      if( !isWorn() ) return nil;
      if(senseUnderClothes(sense)) return nil;
      if(location.contents.length() == 0) return nil;
      foreach(local cur in location.contents)
      {
        if(cur.ofKind(Layered))
        {
          if(cur.conceals(self, sense) && cur.layer > l && cur.isWorn())
          {
            c = cur;
            l = cur.layer;
          }
        }
      }
      return c;
    }

    /* verb handling */
    dobjFor(Wear)
    {
        preCond = [objHeld]
        verify()
        {
            /* make sure the actor isn't already wearing the item */
            if (isWornBy(gActor))
                illogicalAlready(&alreadyWearingMsg);
        }
        check()
        {
          if(blocker(gActor))
          {
            "{You/he} must take off <<blocker(gActor).theName>> before {you/he} can put on <<theName>>. ";
            exit;
          }
          if(clothingSize < gActor.clothingSize)
          {
            tooSmallMsg;
            exit;
          }
          if(clothingSize > gActor.clothingSize)
          {
            tooBigMsg;
            exit;
          }
        }
        action()
        {
            /* make the item worn and describe what happened */
            makeWornBy(gActor);
            wearMsg;
        }
    }

    dobjFor(Doff)
    {
        preCond = [roomToHoldObj]
        verify()
        {
            /* make sure the actor is actually wearing the item */
            if (!isWorn())
                illogicalAlready(&notWearingMsg);
        }
        check()
        {
          if(blocker(location))
          {
            "{You/he} must take off <<blocker(location).theName>> before {you/he} can remove <<theName>>. ";
            exit;
          }
        }
        action()
        {
            /* un-wear the item and describe what happened */
            doffMsg;
            makeWornBy(nil);
            if(!neverCover) noCover = nil;
            gActor.getDropDestination(self, nil)
                .receiveDrop(self, dropTypeDrop);
        }
    }
  dobjFor(Unfasten)
  {
    verify() 
    {
      illogical('{You/he} see{s} no way to unfasten
                {the dobj/him}.  {You/he}\'ll just have to take it off if that\'s what {you/he} want{s} to do. ');
    }
  }
    dobjFor(LookUnder)
    {
        action()
        {
          if(isWorn()) "{You/he} must take off <<theName>> to see what's under it. "; 
          else inherited();
        }
    }
;

/* Manipulable clothing classes for clothing that can be opened, unbuttoned, etc. */
/* These are designed to be mutally exclusive, so for example you can't have an article */
/* of clothing that can be both unbuttoned and pulled down */

/* When "opened," this class uncovers what's beneath it */
/* Defined as a separate class because in some cases author might want layered clothing */
/* to be "openable" is a more traditional sense (like a wearable backpack) */
class OpenableClothing: Layered
    noCover = nil      /* starts closed by default */
    noCoverDesc = "\^<<theName>> <<verbToBe>> opened up. "
    openProp = "open"
    closedProp = "closed"
    openMsg = "{You/he} open{s} <<theName>>. "
    closeMsg = "{You/he} close{s} <<theName>>. "
    alreadyOpenMsg = '{The dobj/he} {is} already open. '
    alreadyClosedMsg = '{The dobj/he} {is} already closed. '
    dobjFor(Open)
    {
      verify()
      {
        if(noCover)
        {
          illogicalAlready(&alreadyOpenMsg);
        }
        else verifyDobjDoff();         /* default: if you can't remove it, you can't open it */
      }
      check()
      {
          if(blocker(location))
          {
            "{You/he} must take off <<blocker(location).theName>> before {you/he} can open <<theName>>. ";
            exit;
          }
      }
      action()
      {
        noCover = true;
        openMsg;
      }
    }
    dobjFor(Close)
    {
      verify()
      {
        if(!noCover)
        {
          illogicalAlready(&alreadyClosedMsg);
        }
        else verifyDobjDoff();         /* default: if you can't remove it, you can't close it */
      }
      check()
      {
          if(blocker(location))
          {
            "{You/he}\'ll have to take off <<blocker(location).theName>> before {you/he} can close <<theName>>. ";
            exit;
          }
      }
      action()
      {
        noCover = nil;
        closeMsg;
      }
    }
    dobjFor(LookUnder) remapTo(Open, self)  /* default is "look under" tries to open article */

;

class LowerableClothing: Layered
    noCover = nil      /* starts closed by default */
    noCoverDesc = "\^<<theName>> <<verbToBe>> pulled down. "
    openProp = "pulled down"
    closedProp = "pulled up"
    openMsg = "{You/he} pull{s} down <<theName>>. "
    closeMsg = "{You/he} pull{s} <<theName>> back up. "
    alreadyOpenMsg = '{The dobj/he} {is} already pulled down. '
    alreadyClosedMsg = '{The dobj/he} {is} already in its usual place. '
    dobjFor(Lower)
    {
      verify()
      {
        if(noCover)
        {
          illogicalAlready(&alreadyOpenMsg);
        }
        else verifyDobjDoff();         /* default: if you can't remove it, you can't open it */
      }
      check()
      {
          if(blocker(location))
          {
            "{You/he} must take off <<blocker(location).theName>> before {you/he} can lower <<theName>>. ";
            exit;
          }
          else checkDobjDoff();     /* default:  if you can't remove it, you can't open it */
      }
      action()
      {
        noCover = true;
        openMsg;
      }
    }
    dobjFor(Raise)
    {
      verify()
      {
        if(!noCover)
        {
          illogicalAlready(&alreadyClosedMsg);
        }
        else verifyDobjDoff();         /* default: if you can't remove it, you can't close it */
      }
      check()
      {
          if(blocker(location))
          {
            "{You/he} must take off <<blocker(location).theName>> before {you/he} can raise <<theName>>. ";
            exit;
          }
      }
      action()
      {
        noCover = nil;
        closeMsg;
      }
    }
    dobjFor(LookUnder) remapTo(Lower, self)
;

class LiftableClothing: Layered
    noCover = nil      /* starts closed by default */
    noCoverDesc = "\^<<theName>> <<verbToBe>> pulled up. "
    openProp = "pulled up"
    closedProp = "pulled down"
    openMsg = "{You/he} pull{s} up <<theName>>. "
    closeMsg = "{You/he} pull{s} <<theName>> back down. "
    alreadyOpenMsg = '{The dobj/he} {is} already pulled up. '
    alreadyClosedMsg = '{The dobj/he} {is} already in its usual place. '
    dobjFor(Raise)
    {
      verify()
      {
        if(noCover)
        {
          illogicalAlready(&alreadyOpenMsg);
        }
        else verifyDobjDoff();         /* default: if you can't remove it, you can't open it */
      }
      check()
      {
          if(blocker(location))
          {
            "{You/he} must take off <<blocker(location).theName>> before {you/he} can raise <<theName>>. ";
            exit;
          }
          else checkDobjDoff();     /* default:  if you can't remove it, you can't open it */
      }
      action()
      {
        noCover = true;
        openMsg;
      }
    }
    dobjFor(Lower)
    {
      verify()
      {
        if(!noCover)
        {
          illogicalAlready(&alreadyClosedMsg);
        }
        else verifyDobjDoff();         /* default: if you can't remove it, you can't close it */
      }
      check()
      {
          if(blocker(location))
          {
            "{You/he} must take off <<blocker(location).theName>> before {you/he} can lower <<theName>>. ";
            exit;
          }
      }
      action()
      {
        noCover = nil;
        closeMsg;
      }
    }
    dobjFor(LookUnder) remapTo(Raise, self)
;

class UnfastenableClothing: Layered
    noCover = nil      /* starts closed by default */
    noCoverDesc = "\^<<theName>> <<verbToBe>> torn. "
    openProp = "unfastened"
    closedProp = "fastened"
    openMsg = "{You/he} unfasten{s} <<theName>>. "
    closeMsg = "{You/he} fasten{s} <<theName>> back up. "
    alreadyOpenMsg = '{The dobj/he} {is} already unfastened. '
    alreadyClosedMsg = '{The dobj/he} {is} already fastened. '
    dobjFor(Unfasten)
    {
      verify()
      {
        if(noCover)
        {
          illogicalAlready(&alreadyOpenMsg);
        }
        else verifyDobjDoff();         /* default: if you can't remove it, you can't open it */
      }
      check()
      {
          if(blocker(location))
          {
            "{You/he} must take off <<blocker(location).theName>> before {you/he} can unfasten <<theName>>. ";
            exit;
          }
          else checkDobjDoff();     /* default:  if you can't remove it, you can't open it */
      }
      action()
      {
        noCover = true;
        openMsg;
      }
    }
    dobjFor(Fasten)
    {
      verify()
      {
        if(!noCover)
        {
          illogicalAlready(&alreadyClosedMsg);
        }
        else verifyDobjDoff();         /* default: if you can't remove it, you can't close it */
      }
      check()
      {
          if(blocker(location))
          {
            "{You/he} must take off <<blocker(location).theName>> before {you/he} can fasten <<theName>>. ";
            exit;
          }
      }
      action()
      {
        noCover = nil;
        closeMsg;
      }
    }
    dobjFor(LookUnder) remapTo(Unfasten, self)
;

class TearableClothing: Layered
    noCover = nil      /* starts closed by default */
    openProp = "torn"
    closedProp = "whole"
    openMsg = "{You/he} tear{s} open <<theName>>. "
    closeMsg = "{You/he} fix{es} <<theName>> back up. "
    alreadyOpenMsg = '{The dobj/he} {is} already torn open. '
    alreadyClosedMsg = '{The dobj/he} {is} already repaired. '
    dobjFor(Tear)
    {
      verify()
      {
        if(noCover)
        {
          illogicalAlready(&alreadyOpenMsg);
        }
        else verifyDobjDoff();         /* default: if you can't remove it, you can't open it */
      }
      check()
      {
          if(blocker(location))
          {
            "{You/he} must take off <<blocker(location).theName>> before {you/he} can unfasten <<theName>>. ";
            exit;
          }
          else checkDobjDoff();     /* default:  if you can't remove it, you can't open it */
      }
      action()
      {
        noCover = true;
        neverCover = true;   /* default is it's torn forever */
        openMsg;
      }
    }
;

/* a BodyArea is an area of the human body that can be covered by clothing */
class BodyArea:  ClothedItem
   clothedDesc = "\^<<theName>> <<verbToBe>> concealed by <<cover.theName>>. "
   bareDesc = "\^<<theName>> <<verbToBe>> unclothed. " 
   desc
   {
     if(cover) clothedDesc;
     else bareDesc;
   }
   
   /* helper description to be used in the desc of the actor wearing it */
   actorClothedDesc = nil   /* no separate description if clothed */
   actorBareDesc = bareDesc
   actorDesc
   {
     if(cover) actorClothedDesc;
     else actorBareDesc;
   }
   uncover(actor)   /* take off outer layer of clothing covering a bodypart */
   {
     local c = cover;
     if(c)
     {
       while(c.blocker(location)) c = c.blocker(location);
       nestedActorAction(actor, Doff, c);
     }
   }
    cover          /* Returns outermost clothing covering this part */
    {
      local c = nil, l = -1;
      if(location.contents.length() == 0) { "Boooo!"; return nil; }
      foreach(local cur in location.contents)
      {
        if(cur.ofKind(Layered))
        {
          if(cur.covers(self) && cur.layer > l && cur.isWorn())
          {
            c = cur;
            l = cur.layer;
          }
        }
      }
      return c;
    }
    concealer(sense)   /* Returns outmost clothing concealing this part from being sensed */
    {
      local c = nil, l = -1;
      if(location.contents.length() == 0) return nil;
      foreach(local cur in location.contents)
      {
        if(cur.ofKind(Layered))
        {
          if(cur.conceals(self, sense) && cur.layer > l && cur.isWorn())
          {
            c = cur;
            l = cur.layer;
          }
        }
      }
      return c;
    }
;

/*** clothing verbs ***/

/* The strip verb tried to take off each piece of clothing sequentially.  Alternatively, you can
*  use the quickUndress method defined above.
*/

DefineIAction(UndressImplicit)
  execAction()
  { 
    if(!gActor.willUndress || gActor.clothingList.length() == 0 )
    {
      "You need to be more specific about what {you/he} should remove. ";
      exit;
    }
    else if( gActor.listClothing.length() == 0 )
    {
      "But {you/he} {are}n't wearing anything! ";
      exit;
    }
    else
    {
      local l = [];
      foreach(local cur in gActor.clothingList)
      {
        l = [cur] + l; //reverses clothing list
      }
      gActor.undressMsg;
      foreach(local cur in l)
      {
        "\b\^<<cur.theName>>:  ";
        tryImplicitAction(Doff, cur);
      }
    }
  }
;


VerbRule(UndressImplicit)
  ( 'undress' | 'strip' | 'get' 'undressed' )
  : UndressImplicitAction
  verbPhrase = 'undress/undressing'
;

DefineTAction(Undress);

VerbRule(Undress)
  ('undress' | 'strip') singleDobj
  : UndressAction
  verbPhrase = 'undress/undressing (what)'
;

DefineIAction(DressImplicit)
  execAction()
  { 
    if(!gActor.willDress || gActor.clothingList.length() == 0 )
    {
      "You need to be more specific about what {you/he} should put on. ";
      exit;
    }
    else
    {
      gActor.dressMsg;
      foreach(local cur in gActor.clothingList)
      {
        "\b\^<<cur.theName>>:  ";
        tryImplicitAction(Wear, cur);
      }
    }
  }
;

VerbRule(DressImplicit)
  ( 'dress' | 'get' 'dressed' )
  : DressImplicitAction
  verbPhrase = 'dress/dressing'
;

DefineTAction(Dress);

VerbRule(Dress)
  ('dress') singleDobj
  : DressAction
  verbPhrase = 'dress/dressing (what)'
;

DefineTAction(Raise);

VerbRule(Raise)
  ('raise' | 'lift' | 'pull' 'up') singleDobj
  | 'pull' singleDobj 'up'
  : RaiseAction
  verbPhrase = 'raise/raising (what)'
;

DefineTAction(Lower);

VerbRule(Lower)
  ('lower' | 'pull' 'down') singleDobj
  | 'pull' singleDobj 'down'
  : LowerAction
  verbPhrase = 'lower/lowering (what)'
;

DefineTAction(Tear);

VerbRule(Tear)
  ('rip' | 'rip' 'open' | 'rip' 'off' | 'tear' | 'tear' 'open' | 'tear' 'off' | 'rip' 'up' | 'tear' 'up') singleDobj
  | ('rip' | 'tear' ) singleDobj ( 'open' | 'off' | 'up' )
  : TearAction
  verbPhrase = 'tear/tearing (what)'
;

replace VerbRule(Unfasten)
    ('unfasten' | 'unbuckle' | 'unzip' | 'unbutton' ) dobjList
    : UnfastenAction
    verbPhrase = 'unfasten/unfastening (what)'
;

replace VerbRule(Fasten)
    ('fasten' | 'buckle' | 'buckle' 'up' | 'zip' | 'zip' 'up' | 'button' | 'button' 'up') dobjList
    : FastenAction
    verbPhrase = 'fasten/fastening (what)'
;

modify Thing
  dobjFor(Raise)
  {
    preCond = [touchObj]
    verify() 
    {
      illogical('{You/he} can\'t raise
                {the dobj/him}. ');
    }
  }
  dobjFor(Lower)
  {
    preCond = [touchObj]
    verify() 
    {
      illogical('{You/he} can\'t lower
                {the dobj/him}. ');
    }
  }
  dobjFor(Tear)
  {
    preCond = [touchObj]
    verify() 
    {
      illogical('Tearing
                {that dobj/him} would serve no purpose. ');
    }
  }
  dobjFor(Undress)
  {
    preCond = [touchObj]
    verify() 
    {
      illogical('{That obj/he} cannot get undressed. ');
    }
  }
  dobjFor(Dress)
  {
    preCond = [touchObj]
    verify() 
    {
      illogical('{That obj/he} cannot get dressed. ');
    }
  }
;


/* objNotWorn is a precondition for some actions.  For some inexplicable
   reason, it only checks to see it the item is worn by the actor, and
   not whether it is worn by someone else.  This replacement checks to
   see if the item is worn by anyone. */
replace objNotWorn: PreCondition
    checkPreCondition(obj, allowImplicit)
    {
        /* if the object isn't being worn, we have nothing to do */
        if (obj == nil || !obj.isWorn)
            return nil;
        
        /* try an implicit 'doff' command */
        if (allowImplicit && tryImplicitAction(Doff, obj))
        {
            /* 
             *   we executed the command - make sure it worked, and abort
             *   if it didn't 
             */
            if (obj.isWorn)
                exit;

            /* tell the caller we executed an implicit command */
            return true;
        }

        /* report the problem and terminate the command */
        reportFailure(&cannotBeWearingMsg, obj);

        /* make it the pronoun */
        gActor.setPronounObj(obj);

        /* abort the command */
        exit;
    }

    /* lower the likelihood rating for anything being worn */
    verifyPreCondition(obj)
    {
        /* if the object is being worn, reduce its likelihood rating */
        if (obj != nil && obj.isWorn)
            logicalRankOrd(80, 'implied doff', 150);
    }
;



/* Various clothing */

class pantyClothing: Layered
  layer = 1
  coverlist = [crotchArea, waistArea]
;

class shirtClothing: Layered
  layer = 2
  coverlist = [chestArea, backArea]
;

class longshirtClothing: Layered
  layer = 2
  coverlist = [chestArea, armsArea, wristArea, backArea]
;

class sweaterClothing: Layered
  layer = 3
  coverlist = [chestArea, armsArea, wristArea, backArea]
;

class braClothing: Layered
  layer = 1
  coverlist = [chestArea]
;

class shortsClothing: Layered
  layer = 2
  coverlist = [crotchArea, waistArea]
;

class pantsClothing: Layered
  layer = 2
  coverlist = [crotchArea, legsArea, waistArea]
;

class dressClothing: Layered
  layer = 2
  coverlist = [crotchArea, legsArea, waistArea, chestArea, backArea]
;

class skirtClothing: Layered
  layer = 2
  coverlist = [crotchArea, legsArea, waistArea]
;

class miniskirtClothing: Layered
  layer = 2
  coverlist = [crotchArea, waistArea]
;

class minidressClothing: Layered
  layer = 2
  coverlist = [crotchArea, waistArea, chestArea, backArea]
;

class coatClothing: Layered
  layer = 4
  coverlist = [chestArea, armsArea, wristArea, backArea]
;

class openCoatClothing: Layered
  layer = 4
  coverlist = [armsArea, wristArea, backArea]
;

class robeClothing: Layered
  layer = 2
  coverlist = [chestArea, backArea, waistArea, armsArea, legsArea, crotchArea]
;

class teddyClothing: Layered
  layer = 1
  coverlist = [chestArea, backArea, waistArea, crotchArea]
;

class suitClothing: Layered
  layer = 3
  coverlist = [chestArea, backArea, waistArea, armsArea, legsArea, crotchArea]
;

class condomClothing: Layered
  layer = 0
  coverlist = [crotchArea]
  noCover = true
  neverCover = true
;

class shoesClothing: Layered
  layer = 2
  coverlist = [feetArea, ankleArea]
  isplural = true
;

class socksClothing: Layered
  layer = 1
  coverlist = [feetArea, ankleArea]
  isplural = true
;

class stockingClothing: Layered
  layer = 1
  coverlist = [feetArea, ankleArea, legsArea]
  isplural = true
;


/*** various body areas ***/

class chestArea: BodyArea
;

class backArea: BodyArea
;

class crotchArea: BodyArea
;

class legsArea: BodyArea
;

class eyesArea: BodyArea
;

class mouthArea: BodyArea
;

class headArea: BodyArea
;

class armsArea: BodyArea
;

class handsArea: BodyArea
;

class feetArea: BodyArea
;

class neckArea: BodyArea
;

class waistArea: BodyArea
;

class wristArea: BodyArea
;

class ankleArea: BodyArea
;

